@@ -342,6 +342,48 @@ def test_transcode_svs_to_zarr(samples_path, tmp_path):
     assert np.array_equal(original, new)
 
 
+def test_transcode_svs_to_pyramid_ome_zarr(samples_path, tmp_path):
+    """Test that we can transcode an J2K SVS to a pyramid OME-Zarr (NGFF)."""
+    reader = readers.Reader.from_file(
+        samples_path
+        / "bfconvert"
+        / (
+            "XYC_-compression_JPEG-2000"
+            "_-tilex_128_-tiley_128"
+            "_-pyramid-scale_2"
+            "_-merge.ome.tiff"
+        )
+    )
+    out_path = tmp_path / (
+        "XYC_-compression_JPEG-2000"
+        "_-tilex_128_-tiley_128_"
+        "-pyramid-scale_2_"
+        "-merge.zarr"
+    )
+    writer = writers.ZarrReaderWriter(
+        path=out_path,
+        tile_size=reader.tile_shape[::-1],
+        dtype=reader.dtype,
+        pyramid_downsamples=[2, 4, 8],
+        ome=True,
+    )
+    writer.transcode_from_reader(reader=reader)
+
+    assert writer.path.exists()
+    assert writer.path.is_dir()
+    assert len(list(writer.path.iterdir())) > 0
+
+    output = zarr.open(writer.path)
+    original = reader[...]
+    new = output[0][...]
+
+    assert np.array_equal(original, new)
+
+    assert "_creator" in writer.zarr.attrs
+    assert "omero" in writer.zarr.attrs
+    assert "multiscales" in writer.zarr.attrs
+
+
 def test_transcode_jpeg_dicom_wsi_to_zarr(samples_path, tmp_path):
     """Test that we can transcode a JPEG compressed DICOM WSI to a Zarr."""
     reader = readers.Reader.from_file(samples_path / "CMU-1-Small-Region")
@@ -414,6 +456,12 @@ def test_cli_jp2_to_tiff(samples_path, tmp_path):
     assert result.exit_code == 0
 
 
+def test_tiff_res_tags(samples_path):
+    """Test that we can read the resolution tags from a TIFF."""
+    reader = readers.Reader.from_file(samples_path / "XYC-half-mpp.tiff")
+    assert reader.microns_per_pixel == (0.5, 0.5)
+
+
 def test_cli_transcode_svs_to_zarr(samples_path, tmp_path):
     """Test the CLI for transcoding."""
     runner = CliRunner()
@@ -428,6 +476,34 @@ def test_cli_transcode_svs_to_zarr(samples_path, tmp_path):
     assert result.exit_code == 0
 
 
+def test_copy_from_reader_timeout(samples_path, tmp_path):
+    """Check that Writer.copy_from_reader raises IOError when timed out."""
+    reader = readers.Reader.from_file(samples_path / "CMU-1-Small-Region.svs")
+    writer = writers.ZarrReaderWriter(
+        path=tmp_path / "CMU-1-Small-Region.zarr",
+        tile_size=reader.tile_shape[::-1],
+        dtype=reader.dtype,
+    )
+    warnings.simplefilter("ignore")
+    with pytest.raises(IOError, match="timed out"):
+        writer.copy_from_reader(reader=reader, timeout=1e-5)
+
+
+def test_cli_convert_timeout(samples_path, tmp_path):
+    """Check that CLI convert raises IOError when reading times out."""
+    runner = CliRunner()
+    with runner.isolated_filesystem(temp_dir=tmp_path) as td:
+        in_path = str(samples_path / "XYC.jp2")
+        out_path = str(Path(td) / "XYC.tiff")
+        warnings.simplefilter("ignore")
+        with pytest.raises(IOError, match="timed out"):
+            runner.invoke(
+                cli.convert,
+                ["-i", in_path, "-o", out_path, "--timeout", "0"],
+                catch_exceptions=False,
+            )
+
+
 def test_help():
     """Test the help output."""
     runner = CliRunner()
