@@ -2,9 +2,9 @@
 
 __author__ = """John Pocock"""
 __email__ = "j.c.pocock@warwick.ac.uk"
-__version__ = "0.2.0"
+__version__ = "0.3.0"
 
-from . import codecs, magic, readers, types, writers
+from . import codecs, magic, metadata, readers, types, writers
 
 __all__ = [
     "codecs",
@@ -12,4 +12,5 @@ __all__ = [
     "readers",
     "types",
     "writers",
+    "metadata",
 ]
