@@ -17,10 +17,9 @@ ext2writer = {
 @click.group(context_settings={"help_option_names": ["-h", "--help"]})
 @click.version_option(wsic.__version__)
 @click.pass_context
-def main(ctx, debug):
+def main(ctx):
     """Console script for wsic."""
     ctx.ensure_object(dict)
-    ctx.obj["DEBUG"] = debug
 
 
 @main.command(no_args_is_help=True)
@@ -61,7 +60,7 @@ def main(ctx, debug):
     "-c",
     "--compression",
     help="The compression to use.",
-    type=click.Choice(["deflate", "webp", "jpeg", "jpeg2000"]),
+    type=click.Choice(["deflate", "webp", "jpeg", "jpeg2000", "blosc"]),
     default="deflate",
 )
 @click.option(
@@ -95,6 +94,13 @@ def main(ctx, debug):
     help="Whether to overwrite the output file.",
     default=False,
 )
+@click.option(
+    "-to",
+    "--timeout",
+    help="Timeout in seconds for reading a tile.",
+    type=float,
+    default=10,
+)
 def convert(
     in_path: str,
     out_path: str,
@@ -107,12 +113,18 @@ def convert(
     microns_per_pixel: float,
     ome: bool,
     overwrite: bool,
+    timeout: float,
 ):
     """Convert a WSI."""
     in_path = Path(in_path)
     out_path = Path(out_path)
     reader = wsic.readers.Reader.from_file(in_path)
-    writer_cls = ext2writer[out_path.suffix]
+    try:
+        writer_cls = ext2writer[out_path.suffix]
+    except KeyError:
+        raise click.BadParameter(
+            f"Unknown file extension {out_path.suffix}", param_hint="out_path"
+        )
     writer = writer_cls(
         out_path,
         shape=reader.shape,
@@ -124,7 +136,9 @@ def convert(
         microns_per_pixel=microns_per_pixel,
         ome=ome,
     )
-    writer.copy_from_reader(reader, read_tile_size=read_tile_size, num_workers=workers)
+    writer.copy_from_reader(
+        reader, read_tile_size=read_tile_size, num_workers=workers, timeout=timeout
+    )
 
 
 @main.command(no_args_is_help=True)
