@@ -1,3 +1,4 @@
+import dataclasses
 import multiprocessing
 import shutil
 import tempfile
@@ -13,6 +14,7 @@ import numpy as np
 import zarr
 
 from wsic.codecs import register_codecs
+from wsic.metadata import ngff
 from wsic.readers import DICOMWSIReader, MultiProcessTileIterator, Reader, TIFFReader
 from wsic.types import PathLike
 from wsic.utils import (
@@ -82,8 +84,9 @@ class Writer(ABC):
         self.compression = compression
         self.compression_level = compression_level or 0
         self.microns_per_pixel = microns_per_pixel
-        self.pyramid_downsamples = pyramid_downsamples or []
-
+        self.pyramid_downsamples = (
+            list(pyramid_downsamples) if pyramid_downsamples else []
+        )
         self.overwrite = overwrite
         self.verbose = verbose
 
@@ -97,6 +100,7 @@ class Writer(ABC):
         read_tile_size: Tuple[int, int] = None,
         yield_tile_size: Tuple[int, int] = None,
         intermediate: zarr.Group = None,
+        timeout: float = 10.0,
     ) -> Iterator:
         """Returns an iterator which returns tiles generated by reader.
 
@@ -124,6 +128,7 @@ class Writer(ABC):
             intermediate=intermediate,
             num_workers=num_workers,
             verbose=self.verbose,
+            timeout=timeout,
         )
 
     def __setitem__(
@@ -138,6 +143,7 @@ class Writer(ABC):
         reader: Reader,
         num_workers: int = 2,
         read_tile_size: Tuple[int, int] = None,
+        timeout: float = 10.0,
     ) -> None:
         """Write pixel data to by copying from a Reader.
 
@@ -149,6 +155,8 @@ class Writer(ABC):
             read_tile_size (Tuple[int, int], optional):
                 Tile size to read. Defaults to None.
                 This will use the tile size of the writer if None.
+            timeout (float, optional):
+                Timeout for workers. Defaults to 10s.
         """
         if self.path.exists() and not self.overwrite:
             raise FileExistsError(f"{self.path} exists and overwrite is False.")
@@ -324,6 +332,7 @@ class JP2Writer(Writer):
         reader: Reader,
         num_workers: int = 2,
         read_tile_size: Optional[Tuple[int, int]] = None,
+        timeout: float = 10.0,
     ) -> None:
         """Write pixel data to by copying from a Reader.
 
@@ -335,11 +344,14 @@ class JP2Writer(Writer):
             read_tile_size (Tuple[int, int], optional):
                 Tile size to read. Defaults to None.
                 This will use the tile size of the writer if None.
+            timeout (float, optional):
+                Timeout for workers. Defaults to 10s.
         """
         super().copy_from_reader(
             reader=reader,
             num_workers=num_workers,
             read_tile_size=read_tile_size,
+            timeout=timeout,
         )
         import glymur
 
@@ -350,6 +362,7 @@ class JP2Writer(Writer):
             reader=reader,
             num_workers=num_workers,
             read_tile_size=read_tile_size or self.tile_size,
+            timeout=timeout,
         )
         reader_tile_iterator = self.level_progress(reader_tile_iterator)
         for tile_writer in jp2.get_tilewriters():
@@ -433,7 +446,6 @@ class TIFFWriter(Writer):
             overwrite=overwrite,
             verbose=verbose,
         )
-        self.image = None
         self.ome = ome
 
     def __setitem__(self, index: Tuple[int, ...], value: np.ndarray) -> None:
@@ -454,6 +466,7 @@ class TIFFWriter(Writer):
         reader: Reader,
         num_workers: int = 2,
         read_tile_size: Optional[Tuple[int, int]] = None,
+        timeout: float = 10.0,
     ) -> None:
         """Write pixel data to by copying from a Reader.
 
@@ -465,11 +478,14 @@ class TIFFWriter(Writer):
             read_tile_size (Tuple[int, int], optional):
                 Tile size to read. Defaults to None.
                 This will use the tile size of the writer if None.
+            timeout (float, optional):
+                Timeout for workers. Defaults to 10s.
         """
         super().copy_from_reader(
             reader=reader,
             num_workers=num_workers,
             read_tile_size=read_tile_size,
+            timeout=timeout,
         )
         import tifffile
 
@@ -492,11 +508,12 @@ class TIFFWriter(Writer):
                 num_workers=num_workers,
                 intermediate=intermediate,
                 read_tile_size=read_tile_size or self.tile_size,
+                timeout=timeout,
             )
             reader_tile_iterator = self.level_progress(reader_tile_iterator)
             # Write baseline (level 0)
             with tifffile.TiffWriter(
-                file=self.path,
+                self.path,
                 bigtiff=True,
                 ome=self.ome,
             ) as tif:
@@ -509,7 +526,7 @@ class TIFFWriter(Writer):
                     metadata["PhysicalSizeY"] = self.microns_per_pixel[1]
 
                 tif.write(
-                    data=reader_tile_iterator,
+                    data=iter(reader_tile_iterator),
                     tile=self.tile_size,
                     shape=reader.shape,
                     dtype=reader.dtype,
@@ -554,7 +571,7 @@ class TIFFWriter(Writer):
                         )
 
                         tif.write(
-                            data=tile_generator,
+                            data=iter(tile_generator),
                             tile=self.tile_size,
                             shape=level_shape,
                             dtype=reader.dtype,
@@ -621,7 +638,6 @@ class ZarrReaderWriter(Writer, Reader):
         if photometric != "rgb":
             warn_unused(photometric)
         warn_unused(microns_per_pixel)
-        warn_unused(ome, ignore_falsey=True)
         super().__init__(
             path=path,
             shape=shape,
@@ -635,6 +651,7 @@ class ZarrReaderWriter(Writer, Reader):
             overwrite=True,
             verbose=verbose,
         )
+        self.ome = ome
         self.overwrite = overwrite
         register_codecs()
         self.compressor = self.get_codec(compression, compression_level)
@@ -644,26 +661,30 @@ class ZarrReaderWriter(Writer, Reader):
             )
 
         self.zarr = None
-        self._init_zarr()
+        self._init_zarr(create=False)
 
-    def _init_zarr(self) -> Optional[zarr.Group]:
+    def _init_zarr(self, create: bool = True) -> Optional[zarr.Group]:
         """Initialize the zarr.
 
         If the zarr already exists, it will be opened. Otherwise, it will be
         created if there is a shape.
 
+        Args:
+            create (bool):
+                Create the zarr if it does not exist.
+
         Returns:
             zarr.Array or zarr.Group:
                 The zarr.
         """
-        # Read and existing zarr
+        # Read an existing zarr
         if self.path.is_dir():
             self.zarr = zarr.open(
                 self.path,
                 mode="r+" if self.overwrite else "r",
             )
         # Create a new zarr group with one array if a shape was given
-        if self.shape is not None:
+        if create:
             self.zarr = zarr.open_group(
                 zarr.NestedDirectoryStore(self.path),
                 mode="a",
@@ -697,7 +718,7 @@ class ZarrReaderWriter(Writer, Reader):
         numcodecs_codecs = {
             "lz4": LZ4,
             "lzma": LZMA,
-            "blosc": Blosc,
+            "blosc": Blosc,  # zarr default
             "blosc-zstd": partial(Blosc, cname="zstd", shuffle=Blosc.BITSHUFFLE),
             "zlib": Zlib,
             "zstd": Zstd,
@@ -747,6 +768,7 @@ class ZarrReaderWriter(Writer, Reader):
         reader: Reader,
         num_workers: int = 2,
         read_tile_size: Optional[Tuple[int, int]] = None,
+        timeout: float = 10.0,
     ) -> None:
         """Write pixel data to by copying from a Reader.
 
@@ -758,11 +780,14 @@ class ZarrReaderWriter(Writer, Reader):
             read_tile_size (Tuple[int, int], optional):
                 Tile size to read. Defaults to None.
                 This will use the tile size of the writer if None.
+            timeout (float, optional):
+                Timeout for workers. Defaults to 10s.
         """
         super().copy_from_reader(
             reader=reader,
             num_workers=num_workers,
             read_tile_size=read_tile_size,
+            timeout=timeout,
         )
 
         # Ensure there is a zarr to write to
@@ -790,6 +815,7 @@ class ZarrReaderWriter(Writer, Reader):
             read_tile_size=read_tile_size,
             yield_tile_size=read_tile_size,
             num_workers=num_workers,
+            timeout=timeout,
         )
         reader_tile_iterator = self.level_progress(reader_tile_iterator)
 
@@ -804,6 +830,52 @@ class ZarrReaderWriter(Writer, Reader):
             level_0[tile_slices(ji, read_tile_size)] = tile
 
         self._build_pyramid()
+        self._write_ome_metadata()
+
+    def _write_ome_metadata(self) -> None:
+        """Write OME-NGFF metadata to the .zattrs file in the root.
+
+        This is based on version 0.4: https://ngff.openmicroscopy.org/0.4/.
+        """
+        if self.ome:
+            multiscales = [
+                ngff.Multiscales(
+                    datasets=[
+                        ngff.Dataset(
+                            path=str(level),
+                            coordinateTransformations=[
+                                ngff.CoordinateTransform(
+                                    "scale",
+                                    [
+                                        1,
+                                        self.microns_per_pixel[0] * downsample,
+                                        self.microns_per_pixel[1] * downsample,
+                                    ],
+                                )
+                            ],
+                        )
+                        for level, downsample in enumerate(
+                            [1] + self.pyramid_downsamples
+                        )
+                    ]
+                    if self.microns_per_pixel is not None
+                    else [],
+                    axes=[
+                        ngff.Axis("y", "space", "micronmeter"),
+                        ngff.Axis("x", "space", "micronmeter"),
+                        ngff.Axis("c", "channel", None),
+                    ],
+                )
+            ]
+            # Convert dataclasses
+            meta_dict = dataclasses.asdict(
+                ngff.Zattrs(multiscales=multiscales),
+                # Exclude None values
+                dict_factory=lambda x: {k: v for (k, v) in x if v is not None},
+            )
+            # Set the attrs
+            for key, value in meta_dict.items():
+                self.zarr.attrs[key] = value
 
     def _build_pyramid(self):
         """Build the pyramid.
@@ -887,12 +959,14 @@ class ZarrReaderWriter(Writer, Reader):
 
         register_codecs()
         codec = self.get_transcode_codec(reader)
+        self.shape = reader.shape
+        self.dtype = reader.dtype
 
         self.zarr = zarr.open_group(zarr.NestedDirectoryStore(self.path))
         self.zarr.create_dataset(
             name="0",
-            shape=reader.shape,
-            dtype=reader.dtype,
+            shape=self.shape,
+            dtype=self.dtype,
             chunks=(*reader.tile_shape, reader.shape[-1]),
             compressor=codec,
         )
@@ -908,6 +982,9 @@ class ZarrReaderWriter(Writer, Reader):
             with open(tile_path, "wb") as file_handle:
                 file_handle.write(tile_bytes)
 
+        self._build_pyramid()
+        self._write_ome_metadata()
+
     @staticmethod
     def get_transcode_codec(reader: TIFFReader) -> Any:
         """Get the codec to use for transcoding.
@@ -1056,6 +1133,7 @@ class ZarrIntermediate(Writer, Reader):
         reader: Reader,
         num_workers: int = 2,
         read_tile_size: Optional[Tuple[int, int]] = None,
+        timeout: float = 10.0,
     ) -> None:
         """Not supported but included for API consistency."""
         raise NotImplementedError()
